import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.util.ArrayList;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;

import fr.esial.seenshare.controlers.CloseSoftware;
import fr.esial.seenshare.models.Albums;
import fr.esial.seenshare.models.CurrentProgressStatus;
import fr.esial.seenshare.models.LoadAlbumsThread;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.views.*;

/**
 * Main class of See&Share
 */
public class SeeNShare extends JFrame {

    private static final long serialVersionUID = 1L;

    private Albums albums;
    private AlbumsView albumsV;
    private InfosView infosV;
    private PreviewView previewV;
    private MenusView menusV;
    private StatusBar statusBar;

    public SeeNShare() {
        super();

        albums = Albums.getInstance();

        menusV = new MenusView(this, albums);
        albums.addObserver(menusV);

        albumsV = new AlbumsView(albums);
        albums.addObserver(albumsV);
        JScrollPane albumsVScroll = new JScrollPane(albumsV,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

        infosV = new InfosView(albums);
        albums.addObserver(infosV);

        previewV = new PreviewView(this);
        albums.addObserver(previewV);

        CurrentProgressStatus status = CurrentProgressStatus.getInstance();
        statusBar = new StatusBar(status);
        status.addObserver(statusBar);

        JPanel pCenter = new JPanel(new BorderLayout());
        pCenter.add(infosV, BorderLayout.NORTH);
        pCenter.add(new JScrollPane(previewV,
                JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
                JScrollPane.HORIZONTAL_SCROLLBAR_NEVER), BorderLayout.CENTER);

        JTabbedPane splitRight = new JTabbedPane();

        ImageIcon iconTab = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_PHY));

        splitRight.addTab(Constants.TAB_ALBUM, iconTab, pCenter, "Current album");
        splitRight.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

        Dimension minimumSize = new Dimension(200, 400);
        splitRight.setMinimumSize(minimumSize);
        albumsVScroll.setMinimumSize(minimumSize);

        JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                true, albumsVScroll, splitRight);

        // Add all main components to the main window
        Container c = getContentPane();
        c.setLayout(new BorderLayout());
        c.add(menusV, BorderLayout.NORTH);
        c.add(splitPane, BorderLayout.CENTER);
        c.add(statusBar, BorderLayout.SOUTH);

        setTitle(Constants.TITLE);
        setPreferredSize(new Dimension(1000, 700));
        ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.IMAGE_ICON));
        setIconImage(icon.getImage());

        // Save albums before exiting app
        CloseSoftware closeController = new CloseSoftware();
        addWindowListener(closeController);
        addKeyListener(closeController);

        pack();
        setLocationRelativeTo(getParent());

        loadData(albums);

        /*
           * Sync albums with disk try { albums.syncAlbums(); } catch (IOException
           * e) { e.printStackTrace(); } catch (ServiceException e) {
           * e.printStackTrace(); } catch (URISyntaxException e) {
           * e.printStackTrace(); }
           */

        setVisible(true);

    }

    /**
     * Load local and online albums
     *
     * @param albums
     */
    private void loadData(Albums albums) {
        new LoadAlbumsThread().start();
    }

    public static void main(String[] args) {
        try {
            UIManager
                    .setLookAndFeel("com.jtattoo.plaf.graphite.GraphiteLookAndFeel");
        } catch (UnsupportedLookAndFeelException e) {
            e.printStackTrace();
        } catch (ClassNotFoundException e) {
            e.printStackTrace();
        } catch (InstantiationException e) {
            e.printStackTrace();
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        }
        new SeeNShare();
    }

}