package fr.esial.seenshare.controlers;

import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.math.BigDecimal;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.text.JTextComponent;

import org.jdesktop.swingx.JXMapKit;
import org.jdesktop.swingx.JXMapKit.DefaultProviders;
import org.jdesktop.swingx.mapviewer.GeoPosition;

import com.google.code.geocoder.Geocoder;
import com.google.code.geocoder.GeocoderRequestBuilder;
import com.google.code.geocoder.model.GeocodeResponse;
import com.google.code.geocoder.model.GeocoderRequest;
import com.google.code.geocoder.model.LatLng;

import fr.esial.seenshare.models.Media;
import fr.esial.seenshare.models.SaveTagsThread;

/**
 * Controller that saves the details of an album
 */
public class AlbumDetailsSave implements FocusListener {

    private Media media;
    private JPanel win;
    private JLabel ext;

    public AlbumDetailsSave(Media media, JPanel win, JLabel ext) {
    	this(media, win);
        this.ext = ext;
    }
    
    public AlbumDetailsSave(Media media, JPanel win) {
        this.media = media;
        this.win = win;
    }

    public void focusLost(FocusEvent e) {
        if (e.getSource() instanceof JTextArea) {
            JTextArea desc = (JTextArea) e.getSource();
            media.setDescription(desc.getText());

        } else {
            JTextField field = (JTextField) e.getSource();

            if (field.getName().equals("title"))
                media.setTitle(field.getText());

            else if (field.getName().equals("tags"))
                new SaveTagsThread(media.getId(), field.getText()).start();

            else if (field.getName().equals("geoloc")) { 
                if (!field.getText().isEmpty()) {
                    String loc[] = field.getText().split("\\s*,\\s*");
                    if (loc != null) {
                        media.setGeoLocation(Double.parseDouble(loc[0]),
                                Double.parseDouble(loc[1]));
                        
                        win.repaint();
                    }
                }
            }
        }

    }

    public void focusGained(FocusEvent e) {
        final JTextComponent field = (JTextComponent) e.getSource();
        final FocusEvent fe = e;

        if (field.getName().equals("geoloc")) {
            final JXMapKit mapKit = new JXMapKit();

            mapKit.setDefaultProvider(DefaultProviders.OpenStreetMaps);
            mapKit.setDataProviderCreditShown(true);
            mapKit.setVisible(true);

            if (!field.getText().matches("^[0-9.-]+\\s*,\\s*[0-9.-]+$"))
                if (media.getGeoLocation() != null)
                    field.setText(media.getGeoLocation().getLatitude() + ","
                            + media.getGeoLocation().getLongitude());

            String[] initCoord = new String[2];
            if (!field.getText().isEmpty())
                initCoord = field.getText().split("\\s*,\\s*");
            else {
                initCoord[0] = "48.857261";
                initCoord[1] = "2.354107";
            }

            mapKit.getMainMap().addMouseListener(
                    new java.awt.event.MouseAdapter() {
                        public void mouseClicked(java.awt.event.MouseEvent evt) {
                            GeoPosition coord = mapKit.getMainMap()
                                    .convertPointToGeoPosition(evt.getPoint());
                            
                            field.setText(coord.getLatitude() + ", "
                                    + coord.getLongitude());

                            final Geocoder geocoder = new Geocoder();
                            GeocoderRequest geocoderRequest = new GeocoderRequestBuilder().setLocation(new LatLng(BigDecimal.valueOf(coord.getLatitude()), BigDecimal.valueOf(coord.getLongitude()))).getGeocoderRequest();
                            GeocodeResponse geocoderResponse = geocoder.geocode(geocoderRequest);
                            
                            if(geocoderResponse.getResults().size() != 0)
                            	ext.setText("<html><p style='width: 200px'><i>" + geocoderResponse.getResults().get(0).getFormattedAddress().replaceAll(",", "<br />") + "</i></p></html>");
                            else
                            	ext.setText("");

                            Component[] components = win.getComponents();

                            mapKit.setVisible(false);

                            field.getFocusListeners()[2].focusLost(fe);

                            for (Component c : components) {
                                if (!(c instanceof JXMapKit))
                                    c.setVisible(true);
                            }
                        }
                    });

            Component[] components = win.getComponents();

            for (Component c : components) {
                c.setVisible(false);
            }

            win.add(mapKit);

            mapKit.setAddressLocation(new GeoPosition(Double
                    .parseDouble(initCoord[0]), Double
                    .parseDouble(initCoord[1])));
        }
    }
}