package fr.esial.seenshare.controlers;

import java.awt.Component;
import java.awt.event.*;

import javax.swing.JFrame;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import fr.esial.seenshare.models.CurrentProgressStatus;
import fr.esial.seenshare.models.Media;

/**
 * Controller that closes a tab when the close button is pressed
 */
public class CloseTabListener implements ActionListener, KeyListener {
    private JFrame win;
    private Media media;
    private boolean ctrlDown;

    public CloseTabListener(JFrame win, Media media) {
        super();
        this.win = win;
        this.media = media;
        this.ctrlDown = false;
    }


    public void closeTab() {
        for (Component c : win.getContentPane().getComponents()) {
            if (c instanceof JSplitPane) {
                JSplitPane splitPane = (JSplitPane) c;
                JTabbedPane splitRight = (JTabbedPane) splitPane
                        .getRightComponent();

                int i = splitRight.indexOfTab(media.getTitle());

                if (i != -1) {

                    if (media.isModified()) {
                        CurrentProgressStatus status = CurrentProgressStatus.getInstance();
                        status.setProgress(0);
                        status.setMessage("Saving image...");

                        media.saveBufImage();

                        status.setProgress(100);
                        status.clearWork();
                    }

                    splitRight.remove(i);
                }
            }
        }
    }

    @Override
    public void actionPerformed(ActionEvent arg0) {
        closeTab();
    }

    @Override
    public void keyTyped(KeyEvent keyEvent) {
        if ((keyEvent.getKeyChar() == 'w') && ctrlDown)
            closeTab();
    }

    @Override
    public void keyPressed(KeyEvent keyEvent) {
        if (keyEvent.isControlDown())
            ctrlDown = true;
    }

    @Override
    public void keyReleased(KeyEvent keyEvent) {
        if (!keyEvent.isControlDown())
            ctrlDown = false;
    }
}
