package fr.esial.seenshare.controlers;

import fr.esial.seenshare.views.PicturePanel;

import javax.swing.JOptionPane;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseWheelEvent;

/**
 * Controller listening for clicks on the photo panel
 */
public class PicturePanelMouseListener extends MouseAdapter {

    boolean isPressed = false;
    boolean isTagging = false;
    boolean isCropping = false;
    int lastX;
    int lastY;

    public void setTagging(boolean tagging) {
        isTagging = tagging;
    }

    public void setCropping(boolean cropping) {
        isCropping = cropping;
    }

    public void setPressed(boolean pressed) {
        isPressed = pressed;
    }

    public void mouseClicked(MouseEvent me) {
        //no event
    }

    public void mouseEntered(MouseEvent me) {
        //no event
    }

    public void mouseExited(MouseEvent me) {
        //no event
    }

    public void mousePressed(MouseEvent me) {
        isPressed = true;

        if (me.isControlDown()) {
            if (me.isShiftDown())
                isCropping = true;
            else
                isTagging = true;
        }
    }

    public void mouseReleased(MouseEvent me) {
        PicturePanel source = (PicturePanel) me.getSource();
        isPressed = false;

        if (isTagging) {
            if (source.getCurrentSel() != null) {
                String name = JOptionPane.showInputDialog("Type a name:");

                if (name != null && !name.isEmpty())
                    source.tagPerson(name);

                source.undoSelectPerson();

            } else {
                String name = source.getTagPersonName(me.getX(), me.getY());

                if (name != null && JOptionPane.showConfirmDialog(null, "Delete the tag for " + name + "?") == JOptionPane.OK_OPTION)
                    ((PicturePanel) me.getSource()).removeTagPerson(name);
            }

            lastX = me.getX();
            lastY = me.getY();

            isTagging = false;
        }
        if (isCropping) {
            if (source.getCurrentSel() != null) {
                int crop = JOptionPane.showConfirmDialog(null, "Crop it?");

                if (crop == JOptionPane.OK_OPTION)
                    source.crop();

                source.undoSelectPerson();
            }

            lastX = me.getX();
            lastY = me.getY();

            isCropping = false;
        }
    }

    /**
     * Move the photo on the panel, or tag or crop
     */
    public void mouseDragged(MouseEvent me) {
        PicturePanel source = (PicturePanel) me.getSource();

        if (isTagging || isCropping) {
            source.selectPerson(this.lastX, this.lastY, me.getX(), me.getY());

        } else {
            source.move(me.getX() - this.lastX, me.getY() - this.lastY);

            lastX = me.getX();
            lastY = me.getY();
        }
    }

    /**
     * Move the photo on the panel
     */
    public void mouseMoved(MouseEvent me) {
        lastX = me.getX();
        lastY = me.getY();
    }

    /**
     * Zoom on the photo (in or out)
     */
    public void mouseWheelMoved(MouseWheelEvent me) {
        PicturePanel source = (PicturePanel) me.getSource();
        source.zoom(me.getWheelRotation() * -1, this.lastX, this.lastY);
    }

}
