package fr.esial.seenshare.controlers;

import fr.esial.seenshare.models.Media;

import java.awt.Graphics;
import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.print.PageFormat;
import java.awt.print.Printable;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;

/**
 * Controller that listen for clicks on the print button
 * Invoke the system printer dialog to select a printer and print a new job
 */
public class PrintMediaListener implements ActionListener {

    private Media media;

    public PrintMediaListener(Media media) {
        this.media = media;
    }

    /**
     * Print the media with system printer
     *
     * @param actionEvent
     */
    @Override
    public void actionPerformed(ActionEvent actionEvent) {
        PrinterJob job = PrinterJob.getPrinterJob();
        final Image img = media.getImage();

        // set the media to print
        job.setPrintable(new Printable() {

            @Override
            public int print(Graphics graphics, PageFormat pageFormat,
                             int pageIndex) throws PrinterException {
                if (pageIndex != 0) {
                    return NO_SUCH_PAGE;
                }
                graphics.drawImage(img, 0, 0, img.getWidth(null),
                        img.getHeight(null), null);
                return PAGE_EXISTS;
            }
        });

        if (job.printDialog()) {
            try {
                job.print();
            } catch (PrinterException ex) {
                ex.printStackTrace();
            }
        }
    }
}
