package fr.esial.seenshare.models;

import com.google.gdata.client.photos.PicasawebService;
import com.google.gdata.util.AuthenticationException;

import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.views.ConnectionDialog;

import javax.swing.*;

/**
 * This class uses Singleton design pattern to handle the authentication to
 * Google: a single instance of the connection exists, is created if null, or
 * returned if exists
 */
public class GoogleService {

    private static PicasawebService googleService = null;
    private static String username = null;
    private static String password = null;

    /**
     * Creates a new connection to googleService with a connection dialog asking for identifiers
     *
     * @throws AuthenticationException
     */
    private GoogleService() {
        new ConnectionDialog();
        if (username != null && password != null) {
            googleService = new PicasawebService(Constants.WEB_SERVICE_NAME);

            try {
                googleService.setUserCredentials(username, password);
            } catch (AuthenticationException e) {
                JOptionPane.showMessageDialog(null, "Error in Google account username or password!", "Account error",
                        JOptionPane.ERROR_MESSAGE);
                googleService = null;
            }
        }
    }

    /**
     * Get the instance of the connection
     *
     * @return connection to picasa
     * @throws AuthenticationException
     */
    public static PicasawebService getGoogleService()
            throws AuthenticationException {
        if (googleService == null) {
            new GoogleService();
        }
        return googleService;
    }

    /**
     * Get the account username
     *
     * @return username
     */
    public static String getUsername() {
        return username;
    }

    /**
     * Get the account password
     *
     * @return password
     */
    public static String getPassword() {
        return password;
    }

    /**
     * Set the account password
     *
     * @param password the account password
     */
    public static void setPassword(String password) {
        GoogleService.password = password;
    }

    /**
     * Set the account username
     *
     * @param username the account username
     */
    public static void setUsername(String username) {
        GoogleService.username = username;
    }

    /**
     * Return if the connection is initiated
     *
     * @return connection is initiated
     */
    public static boolean isNull() {
        return googleService == null;
    }

    /**
     * Reset the connection
     */
    public static void setNull() {
        googleService = null;
    }
}
