package fr.esial.seenshare.models;

import java.awt.Component;
import java.io.IOException;
import java.net.UnknownHostException;
import java.util.concurrent.Semaphore;

import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JSplitPane;
import javax.swing.JTabbedPane;

import com.google.gdata.util.ServiceException;

import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.views.*;

/**
 * Thread that launches the creation of Media previews in JButtons
 * This is one of the slowest operations of the software, this is why it is threaded
 */
public class LoadMediaPreviewsThread extends Thread {
    private PreviewView preview;
    private JFrame win;
    private AlbumsMessage am;
    private Albums albums;
    private Semaphore[] waiting;

    /**
     * Create a new thread
     *
     * @param preview preview where to add media thumbnails
     * @param win     window where to add thumbs
     * @param am      message notified to the observer
     * @param albums  list of albums (useless...)
     * @param waiting semaphore to wait for
     */
    public LoadMediaPreviewsThread(PreviewView preview, JFrame win,
                                   AlbumsMessage am, Albums albums, Semaphore[] waiting) {
        super();
        this.preview = preview;
        this.win = win;
        this.am = am;
        this.albums = albums;
        this.waiting = waiting;
    }

    /**
     * Create a new media preview
     *
     * @param media
     * @param a
     */
    private synchronized void addMediaPreview(Media media, Album a) {
        MediaPreview mediaPreview = null;

        CurrentProgressStatus progressStatus = CurrentProgressStatus
                .getInstance();

        if (media instanceof Photo)
            mediaPreview = new PhotoPreview(media);
        else if (media instanceof Video)
            mediaPreview = new VideoPreview(media);

        // look if album is still selected
        if (a == null || (Albums.getInstance().getSelected() == a && albums.getMediaSearched() == null)) {
            preview.add(mediaPreview);
            win.pack();

            progressStatus.increment();

            if (progressStatus.getProgress() >= 98) {
                progressStatus.clearWork();
            }
        } else
            progressStatus.clearWork();
    }

    @Override
    public void run() {
        super.run();

        JTabbedPane splitRight = null;

        if (am != AlbumsMessage.SEARCH) {
            try {
                waiting[1].acquire();
            } catch (InterruptedException e1) {
                waiting[0].release();
            }
        }

        for (Component c : win.getContentPane().getComponents()) {
            if (c instanceof JSplitPane) {
                JSplitPane splitPane = (JSplitPane) c;
                splitRight = (JTabbedPane) splitPane.getRightComponent();

                if (am != AlbumsMessage.SELECTION_AUTO)
                    splitRight.setSelectedIndex(splitRight.indexOfTab(Constants.TAB_ALBUM));
            }
        }

        if (am == AlbumsMessage.SELECTION || am == AlbumsMessage.SELECTION_AUTO) {
            Album a = albums.getSelected();

            if (am != AlbumsMessage.SELECTION_AUTO) {
                for (Component c : preview.getComponents())
                    preview.remove(c);

                preview.repaint();
            }

            if (a != null) {

                if (a.size() > 0) {
                    CurrentProgressStatus progressStatus = CurrentProgressStatus
                            .getInstance();
                    progressStatus
                            .setMessage("Loading media previews...");

                    if (preview.getComponentCount() == 0) {
                        progressStatus.setProgress(0);
                        if (a.getStatus() == ResourceStatus.ONLINE) {

                            try {
                                progressStatus.setIncrement((double) 100 / a.getOnlineSize());
                            } catch (UnknownHostException e) {
                                ErrorPopup.showNetworkError();
                            } catch (IOException e) {
                                e.printStackTrace();
                            } catch (ServiceException e) {
                                ErrorPopup.showServiceError();
                            }

                        } else {
                            progressStatus.setIncrement((double) 100 / a.size());
                        }
                    }
                }

                for (int i = preview.getComponentCount(); i < a.getMedias().size(); i++) {
                    if (Albums.getInstance().getSelected() == a && albums.getMediaSearched() == null) {
                        Media media = a.getMedias().get(i);
                        addMediaPreview(media, a);
                    } else {
                        waiting[1].release();
                        waiting[0].release();
                        return;
                    }
                }
            }
        } else if (am == AlbumsMessage.SEARCH) {
            for (Component c : preview.getComponents()) {
                preview.remove(c);
            }
            preview.repaint();

            if (albums.getMediaSearched().size() == 0) {
                preview.add(new JLabel("No result."));
            } else {
                CurrentProgressStatus progressStatus = CurrentProgressStatus
                        .getInstance();
                progressStatus
                        .setMessage("Loading media previews...");
                progressStatus.setProgress(0);
                progressStatus.setIncrement(100 / albums
                        .getMediaSearched().size());
            }

            for (Media media : albums.getMediaSearched()) {
                addMediaPreview(media, null);
            }
        }

        if (am != AlbumsMessage.SEARCH) {
            waiting[1].release();
            waiting[0].release();
        }
    }
}
