package fr.esial.seenshare.models;

import com.google.gdata.data.DateTime;
import com.google.gdata.data.PlainTextConstruct;
import com.google.gdata.data.media.MediaFileSource;
import com.google.gdata.data.photos.AlbumEntry;
import com.google.gdata.data.photos.GphotoEntry;
import com.google.gdata.data.photos.PhotoEntry;
import com.google.gdata.data.photos.TagEntry;
import fr.esial.seenshare.utils.ProjectFiles;

import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;

/**
 * Class to be able to write Entry objects in files
 * Convert each attributes of an entry in a serializable object
 */
@SuppressWarnings("serial")
public class SerializableEntry implements Serializable {

    private boolean isAlbum;
    private String title;
    private String description;
    private long updateTime;
    private int updateTimeTzShift;
    private String mediaSource;
    private String visibility;
    private Date date;
    private ArrayList<String> tags;
    private Double latitude;
    private Double longitude;
    private String linkToPicasa;

    /**
     * Create a new serializableEntry to save a resource in a file
     *
     * @param r resource to save (entry to convert)
     */
    @SuppressWarnings("unchecked")
    public SerializableEntry(Resource r) {
        GphotoEntry entry = null;
        if (r instanceof Media) {
            entry = r.getEntry();
            isAlbum = false;
        } else if (r instanceof Album) {
            entry = r.getEntry();
            isAlbum = true;
        }

        if (entry != null) {
            if (entry.getTitle() != null)
                title = entry.getTitle().getPlainText();
            if (entry.getDescription() != null)
                description = entry.getDescription().getPlainText();
            if (entry.getUpdated() != null) {
                updateTime = entry.getUpdated().getValue();
                updateTimeTzShift = entry.getUpdated().getTzShift();
            }

            if (isAlbum) {
                AlbumEntry albumEntry = (AlbumEntry) entry;
                if (albumEntry.getAccess() != null)
                    visibility = albumEntry.getAccess();
                if (albumEntry.getDate() != null)
                    date = albumEntry.getDate();
                if (albumEntry.getGeoLocation() != null) {
                    latitude = albumEntry.getGeoLocation().getLatitude();
                    longitude = albumEntry.getGeoLocation().getLongitude();
                }
                if (albumEntry.getHtmlLink() != null) {
                    linkToPicasa = albumEntry.getHtmlLink().getHref();
                }

            } else {
                PhotoEntry photoEntry = (PhotoEntry) entry;
                if (entry.getMediaSource() != null)
                    mediaSource = entry.getMediaSource().getName();
                if (photoEntry.getGeoLocation() != null) {
                    latitude = photoEntry.getGeoLocation().getLatitude();
                    longitude = photoEntry.getGeoLocation().getLongitude();
                }
            }

            tags = new ArrayList<String>();
            ArrayList<TagEntry> rTags = r.getTags();
            for (TagEntry t : rTags) {
                tags.add(t.getTitle().getPlainText());
            }
        }
    }

    /**
     * Restore a resource entry from a serializableEntry
     *
     * @param r resource where to restore entry
     */
    @SuppressWarnings("unchecked")
    public void setEntry(Resource r) {
        GphotoEntry entry = null;
        if (isAlbum)
            entry = new AlbumEntry();
        else
            entry = new PhotoEntry();

        entry.setTitle(new PlainTextConstruct(title));
        entry.setDescription(new PlainTextConstruct(description));
        entry.setUpdated(new DateTime(updateTime, updateTimeTzShift));
        if (isAlbum) {
            ((AlbumEntry) entry).setAccess(visibility);
            ((AlbumEntry) entry).setDate(date);
            ((AlbumEntry) entry).setGeoLocation(latitude, longitude);
            if (linkToPicasa != null)
                ((AlbumEntry) entry).addHtmlLink(linkToPicasa, null, null);

        } else {
            entry.setMediaSource(new MediaFileSource(new File(mediaSource), ProjectFiles.getFileMimetype(mediaSource)));
            ((PhotoEntry) entry).setGeoLocation(latitude, longitude);
        }

        r.setEntry(entry);

        // tags
        String tagsString = "";
        for (String s : tags) {
            tagsString += s += " ";
        }
        r.setTags(tagsString);
    }
}
