package fr.esial.seenshare.models.tests;

import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;

import com.google.gdata.data.photos.AlbumEntry;
import com.google.gdata.util.ServiceException;

import fr.esial.seenshare.models.*;
import junit.framework.TestCase;

public class AlbumTest extends TestCase {
	Photo photo1, photo2, photo3, photoVideo;
	Video video1, video2, video3, videoPhoto;
	ArrayList<Media> testMedias;
	Album album;
	
	public void sortTestMedias() {
		Collections.sort(testMedias, new Comparator<Media>() {
			
			public int compare(Media o1, Media o2) {
				return o1.getTitle().compareTo(o2.getTitle());
			}
		});
	}

	public void setUp() {
		try {
			album = new Album("testAlbum");
			
			photo1 = new Photo("testPhotoName1", "data/tests/testPhotos/photo1.jpg");
			photo2 = new Photo("testPhotoName2", "data/tests/testPhotos/photo2.jpg");
			photo3 = new Photo("testPhotoName3", "data/tests/testPhotos/photo3.jpg");
			photoVideo = new Photo("testPhotoVideoName", "testPhotoVieoPath");
			
			// TODO : Path videos
			video1 = new Video("testVideoName1", "testVideoPath1");
			video2 = new Video("testVideoName2", "testVideoPath2");
			video3 = new Video("testVideoName3", "testVideoPath3");
			videoPhoto = new Video("testPhotoVideoName", "testPhotoVieoPath");
			
			testMedias = new ArrayList<Media>();

		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}
	
	public void testAddMediaPhotos() {
		System.out.println("testAddMedia : ajout photo1");
		album.addMedia(photo1, false);
		assertEquals(album.getMedias().size(), 1);

		System.out.println("testAddMedia : ajout photo2");
		album.addMedia(photo2, false);
		assertEquals(album.getMedias().size(), 2);

		System.out.println("testAddMedia : ajout photo3");
		album.addMedia(photo3, false);
		assertEquals(album.getMedias().size(), 3);
	}
	
	public void testAddMediaPhotoDouble() {
		System.out.println("testAddMedia : ajout photo en double");
		album.addMedia(photo2, false);
		album.addMedia(photo2, false);
		album.addMedia(photo3, false);
		//assertEquals(album.getMedias().size(), 2);
	}
	
	public void testAddMediaVideos() {
		System.out.println("testAddMedia : ajout video1");
		album.addMedia(video1, false);
		assertEquals(album.getMedias().size(), 1);
		
		System.out.println("testAddMedia : ajout video2");
		album.addMedia(video3, false);
		assertEquals(album.getMedias().size(), 2);
		
		System.out.println("testAddMedia : ajout video3");
		album.addMedia(video3, false);
		assertEquals(album.getMedias().size(), 3);
	}
		
	public void testAddMediaVideoDouble() {
		System.out.println("testAddMedia : ajout video en double");
		album.addMedia(video1, false);
		album.addMedia(video2, false);
		album.addMedia(video2, false);
		//assertEquals(album.getMedias().size(), 2);
	}
	
	public void testAddMediaMelangePhotosVideos() {
		System.out.println("testAddMedia : ajout melange photo1 et video1");
		album.addMedia(photo1, false);
		album.addMedia(video1, false);
		assertEquals(album.getMedias().size(), 2);
		
		System.out.println("testAddMedia : ajout melange photo2 et video2");
		album.addMedia(photo2, false);
		album.addMedia(video2, false);
		assertEquals(album.getMedias().size(), 4);
		
		System.out.println("testAddMedia : ajout melange photo3 et video3");
		album.addMedia(photo3, false);
		album.addMedia(video3, false);
		assertEquals(album.getMedias().size(), 6);
	}
	
	public void testAddMediaMelangePhotoVideoDouble() {
		System.out.println("testAddMedia : ajout video avec le meme nom qu'une photo");
		album.addMedia(video1, false);
		album.addMedia(photo1, false);
		album.addMedia(photoVideo, false);
		album.addMedia(videoPhoto, false);
		//assertEquals(album.getMedias().size(), 3);
	}

	public void testRemoveMediaPhotos() {
		System.out.println("testRemoveMedia : ajout et suppression 1 photo");
		album.addMedia(photo1, false);
		album.removeMedia(photo1);
		assertEquals(album.getMedias().size(), 0);
		
		System.out.println("testRemoveMedia : ajout 2 photos et suppression 1 photo");
		album.addMedia(photo1, false);
		album.addMedia(photo2, false);
		album.removeMedia(photo1);
		assertEquals(album.getMedias().size(), 1);
		
		System.out.println("testRemoveMedia : ajout 3 photos et suppression 1 photo");
		album.addMedia(photo1, false);
		album.addMedia(photo3, false);
		album.removeMedia(photo1);
		assertEquals(album.getMedias().size(), 2);
	}	
	
	public void testRemoveMediaVideos() {
		System.out.println("testRemoveMedia : ajout et suppression 1 video");
		album.addMedia(video1, false);
		album.removeMedia(video1);
		assertEquals(album.getMedias().size(), 0);
		
		System.out.println("testRemoveMedia : ajout 2 videos et suppression 1 video");
		album.addMedia(video1, false);
		album.addMedia(video2, false);
		album.removeMedia(video1);
		assertEquals(album.getMedias().size(), 1);
		
		System.out.println("testRemoveMedia : ajout 3 videos et suppression 1 video");
		album.addMedia(video1, false);
		album.addMedia(video3, false);
		album.removeMedia(video1);
		assertEquals(album.getMedias().size(), 2);
	}
	
	public void testRemoveMediaMelangePhotosVideos() {
		System.out.println("testRemoveMedia : ajout 1 photo/1 video et suppression 1 photo");
		album.addMedia(video1, false);
		album.addMedia(photo1, false);
		album.removeMedia(photo1);
		assertEquals(album.getMedias().size(), 1);
		
		System.out.println("testRemoveMedia : ajout 1 photo/1 video et suppression 1 video");
		album.addMedia(photo1, false);
		album.removeMedia(video1);
		assertEquals(album.getMedias().size(), 1);
	}
	
	public void testRemoveMediaPhotoVideoNonAjoutee() {
		album.addMedia(photo1, false);
		album.addMedia(photo2, false);
		album.addMedia(video1, false);

		System.out.println("testRemoveMedia : suppression d'une photo non-ajoutee");
		album.removeMedia(photo3);
		assertEquals(album.getMedias().size(), 3);
		
		System.out.println("testRemoveMedia : suppression d'une video non-ajoutee");
		album.removeMedia(video2);
		assertEquals(album.getMedias().size(), 3);
	}

	public void testSizeAucunePhotoVideo() {
		System.out.println("testSize : aucune photo ni video");
		assertEquals(album.size(), 0);
	}
		
	public void testSizePhotos() {
		System.out.println("testSize : ajout photo1");
		album.addMedia(photo1, false);
		assertEquals(album.size(), 1);
		
		System.out.println("testSize : ajout photo2");
		album.addMedia(photo2, false);
		assertEquals(album.size(), 2);
		
		System.out.println("testSize : ajout photo3");
		album.addMedia(photo3, false);
		assertEquals(album.size(), 3);
	}
	
	public void testSizeVideos() {
		System.out.println("testSize : ajout video1");
		album.addMedia(video1, false);
		assertEquals(album.size(), 1);
		
		System.out.println("testSize : ajout video2");
		album.addMedia(video2, false);
		assertEquals(album.size(), 2);
		
		System.out.println("testSize : ajout video3");
		album.addMedia(video3, false);
		assertEquals(album.size(), 3);
	}
	
	public void testSizeMelangePhotosVideos() {
		System.out.println("testSize : ajout melange photo1 avec video1");
		album.addMedia(photo1, false);
		album.addMedia(video1, false);
		assertEquals(album.size(), 2);
		
		System.out.println("testSize : ajout melange photo2 avec video2");
		album.addMedia(photo2, false);
		album.addMedia(video2, false);
		assertEquals(album.size(), 4);
		
		System.out.println("testSize : ajout melange photo3 avec video3");
		album.addMedia(photo3, false);
		album.addMedia(video3, false);
		assertEquals(album.size(), 6);
	}
	
	public void testSizeMelangePhotosVideosAvecSuppressions() {
		
		System.out.println("testSize : suppression photo1");
		album.addMedia(photo1, false);
		album.addMedia(photo2, false);
		album.addMedia(video1, false);
		album.addMedia(video2, false);
		album.removeMedia(photo1);
		assertEquals(album.size(), 3);
		
		System.out.println("testSize : suppression video1");
		album.removeMedia(video1);
		assertEquals(album.size(), 2);
	}

	public void testGetMediasAucunePhotoVideo() {
		System.out.println("testGetMedias : aucune photo ni video");
		assertEquals(album.getMedias(), testMedias);
	}
	
	public void testGetMediasPhotos() {
		System.out.println("testGetMedias : avec 1 photo");
		testMedias.add(photo1);
		album.addMedia(photo1, false);
		assertEquals(album.getMedias(), testMedias);
		
		System.out.println("testGetMedias : avec 2 photos");
		testMedias.add(photo2);
		album.addMedia(photo2, false);
		assertEquals(album.getMedias(), testMedias);
		
		System.out.println("testGetMedias : avec 3 photos");
		testMedias.add(photo3);
		album.addMedia(photo3, false);
		assertEquals(album.getMedias(), testMedias);
	}
		
	public void testGetMediasVideos() {
		System.out.println("testGetMedias : avec 1 video");
		testMedias.add(video1);
		album.addMedia(video1, false);
		assertEquals(album.getMedias(), testMedias);
		
		System.out.println("testGetMedias : avec 2 videos");
		testMedias.add(video2);
		album.addMedia(video2, false);
		assertEquals(album.getMedias(), testMedias);
		
		System.out.println("testGetMedias : avec 3 videos");
		testMedias.add(video3);
		album.addMedia(video3, false);
		assertEquals(album.getMedias(), testMedias);
	}
	
	public void testGetMediasMelangePhotosVideos() {
		System.out.println("testGetMedias : avec 1 video et 1 photo");
		
		testMedias.add(video1);
		testMedias.add(photo1);
		album.addMedia(video1, false);
		album.addMedia(photo1, false);
		
		sortTestMedias();
		assertEquals(album.getMedias(), testMedias);
		
		System.out.println("testGetMedias : avec 2 videos et 2 photos");
		testMedias.add(video2);
		testMedias.add(photo2);
		album.addMedia(video2, false);
		album.addMedia(photo2, false);
		
		sortTestMedias();
		assertEquals(album.getMedias(), testMedias);
		
		System.out.println("testGetMedias : avec 3 videos et 3 photos");
		testMedias.add(video3);
		testMedias.add(photo3);
		album.addMedia(video3, false);
		album.addMedia(photo3, false);
		
		sortTestMedias();
		assertEquals(album.getMedias(), testMedias);
	}

	public void testGetEntry() {
		AlbumEntry entry = album.getEntry();
		
		System.out.println("testGetEntry : test de la bonne restitution avec le title");
		assertEquals("testAlbum", entry.getTitle().getPlainText());
	}

	public void testSetEntryAlbumEntry() {
		Album album2;
		AlbumEntry entry;

		System.out.println("testSetEntryAlbumEntry : test du remplacement d'un entry");
		
		try {
			album2 = new Album("testAlbum2");
			album.setEntry(album2.getEntry());
			entry = album.getEntry();
			assertEquals("testAlbum2", entry.getTitle().getPlainText());
			
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}

	public void testSynchronizeLocalVersDistant() {
		Albums albums = Albums.getNewInstance();
		
		System.out.println("testSynchronize : test de la synchro en ligne (local vers distant)");

		//try {
			album.addMedia(photo1, true);
			album.addMedia(photo2, true);
			album.addMedia(photo3, true);
	
			System.out.println("nb:" + album.size());
			
			/*			album.synchronizeNewAlbum();
			albums.loadOnlineAlbums();

			assertTrue(albums.getAlbums().keySet().contains("testAlbum"));
			assertEquals(3, albums.getAlbums().get("testAlbum").size());

			album.delete();
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}*/
	}
	
	public void testSynchronizeDistantVersLocal() {
		System.out.println("testSynchronize : test de la synchro en ligne (distant vers local)");

		try {
			Album album2 = new Album("testAlbum");
			
			album.addMedia(photo1, false);
			album.addMedia(photo2, false);
			album.addMedia(photo3, false);
		
			/*			album.synchronizeNewAlbum();
			album2.synchronizeNewAlbum();
			
			sortTestMedias();

			assertEquals(photo2, album2.getMedias().get(1));
			assertEquals(3, album2.size());

			album.delete();
			
		} catch (IOException e) {
			e.printStackTrace();
		} catch (ServiceException e) {
			e.printStackTrace();*/
		} catch (URISyntaxException e) {
			e.printStackTrace();
		}
	}
}