package fr.esial.seenshare.views;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.math.BigDecimal;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import com.google.code.geocoder.Geocoder;
import com.google.code.geocoder.GeocoderRequestBuilder;
import com.google.code.geocoder.model.GeocodeResponse;
import com.google.code.geocoder.model.GeocoderRequest;
import com.google.code.geocoder.model.LatLng;
import com.google.gdata.data.geo.Point;
import com.google.gdata.data.photos.PhotoEntry;
import com.google.gdata.data.photos.TagEntry;

import fr.esial.seenshare.controlers.AlbumDetailsSave;
import fr.esial.seenshare.models.Media;

@SuppressWarnings("serial")
/**
 * Panel that displays the details on a media : title, description, tags, geolocation
 */
public class DetailsPanel extends JPanel {

    private Media media;

    public DetailsPanel(Media m, JPanel win) {
        super();

        media = m;

        StringBuilder tags = new StringBuilder();

        for (TagEntry t : media.getTags())
            tags.append(t.getTitle().getPlainText() + " ");

        JTextField titleField = new JTextField(media.getTitle());
        JTextArea descField = new JTextArea(media.getDescription());
        JTextField tagsField = new JTextField(tags.toString());
        

       	JTextField geolocField = new JTextField();
       	JLabel geolocLabel = new JLabel();
        boolean geoloc = true;
        
        if (media.getGeoLocation() != null) {
        	try {
	            geolocField.setText(media.getGeoLocation().getLatitude() + "," + media.getGeoLocation().getLongitude());
	            
	            final Geocoder geocoder = new Geocoder();
	            Point geoLoc = media.getGeoLocation();
            	Double check = geoLoc.getLatitude();
	            
	            if (check != null) {
		            GeocoderRequest geocoderRequest = new GeocoderRequestBuilder().setLocation(new LatLng(BigDecimal.valueOf(geoLoc.getLatitude()), BigDecimal.valueOf(geoLoc.getLongitude()))).getGeocoderRequest();
		            GeocodeResponse geocoderResponse = geocoder.geocode(geocoderRequest);
		            
		            if(geocoderResponse.getResults().size() != 0)
		            	geolocLabel.setText("<html><p style='width: 200px'><i>" + geocoderResponse.getResults().get(0).getFormattedAddress().replaceAll(",", "<br />") + "</i></p></html>");
		            else
		            	geolocLabel.setText("");
	            }
        	} catch (Exception e) {
        		geoloc = false;
        		// Le service de géolocalisation ne repond pas >_<
        	}
        }

        AlbumDetailsSave controler = new AlbumDetailsSave(media, win);

        titleField.addFocusListener(controler);
        descField.addFocusListener(controler);
        if (geoloc) {
        	geolocField.addFocusListener(new AlbumDetailsSave(media, win, geolocLabel));
        }
        tagsField.addFocusListener(controler);
        tagsField.setToolTipText("Space separated values");

        titleField.setName("title");
        descField.setName("desc");
        tagsField.setName("tags");
        if (geoloc) {
        	geolocField.setName("geoloc");
        }

        setPreferredSize(new Dimension(210, -1));
        setLayout(new ModifiedFlowLayout(FlowLayout.LEFT));

        JLabel label = new JLabel("Title:");
        label.setFont(new Font("SansSerif", Font.BOLD, 12));
        add(label);
        add(titleField);
        titleField.setPreferredSize(new Dimension(200, 23));

        label = new JLabel("Tags:");
        label.setFont(new Font("SansSerif", Font.BOLD, 12));
        add(label);
        add(tagsField);
        tagsField.setPreferredSize(new Dimension(200, 23));


        if (geoloc) {
	        label = new JLabel("Localisation:");
	        label.setFont(new Font("SansSerif", Font.BOLD, 12));
	        add(label);
	        add(geolocField);
	        add(geolocLabel);
	        geolocField.setPreferredSize(new Dimension(200, 23));
        }
        
        label = new JLabel("Description:");
        label.setFont(new Font("SansSerif", Font.BOLD, 12));
        add(label);
        add(descField);
        descField.setPreferredSize(new Dimension(200, 270));
        descField.setLineWrap(true);

        setVisible(false);
    }

    public void displayDetails() {
        setVisible(!isVisible());
    }
}