package fr.esial.seenshare.views;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;

import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JPanel;

import fr.esial.seenshare.models.DiaporamaIterator;
import fr.esial.seenshare.models.Media;
import fr.esial.seenshare.utils.Constants;

/**
 * Class that displays a slideshow of an album
 */
public class Diaporama extends JFrame {

	private static final long serialVersionUID = 1L;

	private Thread thread;
	private boolean interrupted;
	private JPanel pnlImage;
	private DiaporamaIterator di;
	private Media m;

	public Diaporama(ArrayList<Media> a, String title) {
		super(Constants.TITLE_DIAPORAMA + title);
		di = new DiaporamaIterator(a);
		m = di.next();
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		setResizable(false);
		setUndecorated(true);
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		setBounds(0, 0, screenSize.width, screenSize.height);
		ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
				Constants.IMAGE_ICON));
		setIconImage(icon.getImage());
		setLocationRelativeTo(getParent());

		Container c = getContentPane();
		c.setLayout(new BorderLayout());

		interrupted = true;
		thread = null;

		pnlImage = new JPanel() {

			private static final long serialVersionUID = 1L;

			public void paintComponent(Graphics g) {
				super.paintComponent(g);
				if (m != null) {
					Graphics2D g2d = (Graphics2D) g;
					Image img = m.getImage();
					double ratio = 1;
					double w = img.getWidth(null);
					double h = img.getHeight(null);
					if (img.getHeight(null) > pnlImage.getHeight()
							|| img.getWidth(null) > pnlImage.getWidth()) {
						double size = Math.min(pnlImage.getHeight(), pnlImage
								.getWidth());
						w = img.getWidth(null);
						h = img.getHeight(null);

						if (img.getWidth(null) >= img.getHeight(null)) {
							ratio = size / w;
						} else {
							ratio = size / h;
						}
					}
					double x = (int) (pnlImage.getWidth() - w * ratio) / 2;
					double y = (int) (pnlImage.getHeight() - h * ratio) / 2;
					g2d.drawImage(img, (int) x, (int) y, (int) (w * ratio),
							(int) (h * ratio), this);
					g2d.setColor(new Color(100, 100, 100, 150));
					g2d.drawRoundRect(10, 10, 170, 80, 5, 5);
					g2d.setColor(new Color(100, 100, 100, 100));
					g2d.fillRoundRect(10, 10, 170, 80, 5, 5);
					g2d.setColor(new Color(200, 200, 200, 150));
					g2d.drawString("Next [D] [K] [->]", 20, 30);
					g2d.drawString("Previous [Q] [A] [J] [<-]", 20, 45);
					g2d.drawString("Play/Pause [Pause] [P]", 20, 60);
					g2d.drawString("Quit [Escap]", 20, 75);
				}
			}
		};
		pnlImage.setBackground(Color.BLACK);

		c.add(pnlImage, BorderLayout.CENTER);

		addMouseListener(new MouseAdapter() {

			public void mouseClicked(MouseEvent e) {
				actionNext();
			}
		});

		addKeyListener(new KeyAdapter() {

			@Override
			public void keyPressed(KeyEvent e) {
				switch (e.getKeyCode()) {
				case KeyEvent.VK_RIGHT:
				case KeyEvent.VK_D:
				case KeyEvent.VK_K:
					actionNext();
					break;
				case KeyEvent.VK_LEFT:
				case KeyEvent.VK_Q:
				case KeyEvent.VK_A:
				case KeyEvent.VK_J:
					actionPrev();
					break;
				case KeyEvent.VK_PAUSE:
				case KeyEvent.VK_P:
					actionDiap();
					break;
				case KeyEvent.VK_ESCAPE:
					actionClose();
					break;
				}
			}
		});
		setFocusable(true);

		setVisible(true);

		actionDiap();
	}

	private void actionPrev() {
		m = di.previous();
		pnlImage.repaint();
	}

	private void actionNext() {
		m = di.next();
		pnlImage.repaint();
	}

	private void actionDiap() {
		if (thread == null || thread != null && !thread.isAlive()) {
			thread = new Thread() {

				public void run() {
					interrupted = false;
					while (di.hasNext()) {
						long t = System.currentTimeMillis();
						while (System.currentTimeMillis() - t < 4000) {
							try {
								Thread.sleep(100);
								if (interrupted) {
									return;
								}
							} catch (InterruptedException e) {
								e.printStackTrace();
							}
						}
						actionNext();
						if (!di.hasNext()) {
							break;
						}
					}
				}
			};
			thread.start();
		} else {
			interrupted = true;
		}
	}

	private void actionClose() {
		interrupted = true;
		while (thread != null && thread.isAlive()) {
			try {
				Thread.sleep(100);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		}
		dispose();
	}
}
