package fr.esial.seenshare.views;

import java.awt.Component;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Observable;
import java.util.Observer;
import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;

import fr.esial.seenshare.controlers.AddMediaListener;
import fr.esial.seenshare.models.*;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.utils.JButtonIcon;

/**
 * View that displays the info about an album : title, update time, visibility
 * Some buttons are displayed to control albums: add medias, launch a slideshow...
 */
public class InfosView extends JPanel implements Observer {

    private static final long serialVersionUID = 1L;
    private Albums albums;
    private JLabel lblTitle;
    private JLabel lblStatus;
    private JLabel lblDate;
    private JButton syncButton;
    private JButton diapButton;
    private JButton addMediaButton;
    private JComboBox visibilityComboBox;
    private JPanel pnlVisibility;
    private URLLabel linkToPicasa;

    /**
     * Create a new view to display infosabout an album
     *
     * @param a album to display
     */
    public InfosView(Albums a) {
        super(new GridLayout(1, 0, 5, 5));
        albums = a;
        setBorder(BorderFactory.createTitledBorder("Informations"));
        lblTitle = new JLabel();
        lblTitle.setFont(new Font("SansSerif", Font.BOLD, 18));
        lblStatus = new JLabel();
        lblDate = new JLabel();
        linkToPicasa = new URLLabel(null, "  Open in PicasaWeb");

        ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_BTN_SYNC));
        syncButton = new JButtonIcon("Sync", icon);
        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_BTN_SLIDESHOW));
        diapButton = new JButtonIcon("Slideshow", icon);
        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_BTN_ADD));
        addMediaButton = new JButtonIcon("Add", icon);

        // Add a combobox to select album access
        String[] accessTypes = {"Private", "Protected", "Public"};
        visibilityComboBox = new JComboBox(accessTypes);

        syncButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent arg0) {
                if (albums.getSelected() != null)
                    new SyncNewAlbumThread(albums.getSelected()).start();
            }

        });

        diapButton.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent arg0) {
                Albums a = Albums.getInstance();
                if (a.getMediaSearched() != null) {
                    new Diaporama(a.getMediaSearched(), "Search results");
                } else {
                    new Diaporama(a.getSelected().getMedias(), a.getSelected()
                            .getTitle());
                }
            }

        });

        addMediaButton.addActionListener(new AddMediaListener());

        visibilityComboBox.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String access = (String) ((JComboBox) e.getSource())
                        .getSelectedItem();
                Albums.getInstance().getSelected().setAccess(access.toLowerCase());
            }
        });

        JPanel pnlTitle = new JPanel(new GridLayout(0, 1));
        pnlTitle.add(lblTitle);
        pnlTitle.add(lblStatus);
        pnlTitle.add(lblDate);
        pnlTitle.add(linkToPicasa);

        pnlVisibility = new JPanel(new GridLayout(0, 1, 5, 5));
        JLabel lblVisibility = new JLabel("Visibility:");
        lblVisibility.setFont(new Font("SansSerif", Font.BOLD, 12));
        pnlVisibility.add(lblVisibility);
        pnlVisibility.add(visibilityComboBox);

        JPanel pnlButton = new JPanel();
        pnlButton.add(pnlVisibility);
        pnlButton.add(syncButton);
        pnlButton.add(diapButton);
        pnlButton.add(addMediaButton);

        add(pnlTitle);
        add(pnlButton);

        for (Component c : getComponents()) {
            c.setVisible(false);
        }
    }

    @Override
    public void update(Observable o, Object arg) {

        if (o != null && o instanceof Albums) {
            Albums albums = (Albums) o;
            if (arg != null && arg instanceof AlbumsMessage) {
                AlbumsMessage am = (AlbumsMessage) arg;
                if (am == AlbumsMessage.SELECTION
                        || am == AlbumsMessage.SELECTION_AUTO) {
                    lblStatus.setVisible(true);
                    lblDate.setVisible(true);
                    addMediaButton.setVisible(true);

                    Album a = albums.getSelected();
                    if (a != null) {
                        lblTitle.setText(" " + albums.getSelectedName());
                        String status;
                        switch (a.getStatus()) {
                            default:
                            case LOCAL:
                                status = "Local";
                                syncButton.setVisible(true);
                                pnlVisibility.setVisible(false);
                                linkToPicasa.setVisible(false);
                                break;
                            case ONLINE:
                                status = "Online";
                                syncButton.setVisible(true);
                                pnlVisibility.setVisible(true);
                                linkToPicasa.setVisible(true);
                                break;
                            case SYNCHRONIZED:
                                status = "Synchronized";
                                syncButton.setVisible(false);
                                pnlVisibility.setVisible(true);
                                linkToPicasa.setVisible(true);
                                break;
                        }
                        lblStatus.setText("  Status: " + status);
                        lblDate.setText("  Last update: "
                                + a.getEntry().getUpdated().toUiString());
                        setLinkToPicasa(a, linkToPicasa);

                        selectVisibilityInBox(a.getAccess(), visibilityComboBox);
                        if (a.size() > 0) {
                            diapButton.setEnabled(true);
                        } else {
                            diapButton.setEnabled(false);
                        }
                        for (Component c : getComponents()) {
                            c.setVisible(true);
                        }
                    } else {
                        for (Component c : getComponents()) {
                            c.setVisible(false);
                        }
                    }
                } else if (am == AlbumsMessage.SEARCH) {
                    lblStatus.setVisible(false);
                    addMediaButton.setVisible(false);
                    syncButton.setVisible(false);
                    pnlVisibility.setVisible(false);
                    linkToPicasa.setVisible(false);

                    lblTitle.setText("Search results");

                    repaint();
                }
            }
        }

    }

    /**
     * Select current visibility of an album in the combobox
     *
     * @param visibility visibility to select in the box
     * @param box        combobox component
     */
    public void selectVisibilityInBox(String visibility, JComboBox box) {
        if (visibility.compareTo("private") == 0)
            box.setSelectedIndex(0);
        else if (visibility.compareTo("protected") == 0)
            box.setSelectedIndex(1);
        else if (visibility.compareTo("public") == 0)
            box.setSelectedIndex(2);
    }

    /**
     * Initialize the link to the online album on picasa website
     *
     * @param a        album to link to
     * @param urlLabel link component
     */
    public void setLinkToPicasa(Album a, URLLabel urlLabel) {
        if (a.getStatus() == ResourceStatus.ONLINE || a.getStatus() == ResourceStatus.SYNCHRONIZED) {
            try {
                linkToPicasa.setUrl(new URL(a.getEntry().getHtmlLink().getHref()));
            } catch (MalformedURLException e) {
                e.printStackTrace();
            }
        }
    }
}
