package fr.esial.seenshare.views;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.JTextPane;
import javax.swing.JSplitPane;

import fr.esial.seenshare.controlers.CloseTabListener;
import fr.esial.seenshare.controlers.PrintMediaListener;
import fr.esial.seenshare.models.Media;
import fr.esial.seenshare.models.Photo;
import fr.esial.seenshare.utils.Constants;

/**
 * View displaying a media
 */
public abstract class MediaPanel extends JPanel {

    private static final long serialVersionUID = 1L;

    public JPanel buttonTopPanel;
    public JPanel bottomPanel;
    public JTextPane bottomInfos;
    public DetailsPanel detailsPanel;
    public JFrame win;
    public static Media media = null;

    public MediaPanel(Media m, JFrame w) {
        super();

        win = w;
        media = m;

        // switch the right side of the window
        for (Component c : win.getContentPane().getComponents()) {
            if (c instanceof JSplitPane) {
                JSplitPane splitPane = (JSplitPane) c;
                JTabbedPane splitRight = (JTabbedPane) splitPane.getRightComponent();

                if (splitRight.indexOfTab(media.getTitle()) == -1) {
                    ImageIcon iconTab = new ImageIcon(getClass().getClassLoader().getResource(
                            Constants.ICON_IMG));
                    splitRight.addTab(media.getTitle(), iconTab, this, null);
                }

                splitRight.setSelectedIndex(splitRight.indexOfTab(media.getTitle()));
            }
        }

        setLayout(new BorderLayout());

        //Setting up buttons top panel
        buttonTopPanel = new JPanel();
        buttonTopPanel.setLayout(new BorderLayout());
        add(buttonTopPanel, BorderLayout.NORTH);

        // Setting up details panel, hidden by default
        detailsPanel = new DetailsPanel(media, this);
        add(detailsPanel, BorderLayout.EAST);

        //Bottom panel, displays some infos about the current media, and a button to display details
        bottomPanel = new JPanel(new BorderLayout());

        bottomInfos = new JTextPane();
        bottomInfos.setText("Loading...");
        bottomInfos.setOpaque(false);
        bottomInfos.setEditable(false);

        ImageIcon iconDetails = new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_DETAILS));
        JButton showDetails = new JButton(iconDetails);
        showDetails.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                detailsPanel.displayDetails();
            }
        });

        JPanel buttons = new JPanel();

        if (media instanceof Photo) {
            ImageIcon iconPrint = new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_PRINT));
            JButton showPrint = new JButton(iconPrint);

            showPrint.addActionListener(new PrintMediaListener(media));

            buttons.add(showPrint);
        }

        buttons.add(showDetails);
        bottomPanel.add(bottomInfos, BorderLayout.WEST);
        bottomPanel.add(buttons, BorderLayout.EAST);
        add(bottomPanel, BorderLayout.SOUTH);

        addKeyListener(new CloseTabListener(win, media));

        win.pack();
    }
}
