package fr.esial.seenshare.views;

import java.awt.Cursor;
import java.awt.Image;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;

import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.TransferHandler;

import fr.esial.seenshare.models.Albums;
import fr.esial.seenshare.models.AlbumsMessage;
import fr.esial.seenshare.models.Media;
import fr.esial.seenshare.models.ResourceStatus;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.utils.JButtonIcon;

/**
 * View that loads a preview of a media in a jbutton
 * It resizes the photos to fit in a sized button or display a default image for videos
 * This button is draggable to execute operations on medias (copy/move)
 */
public class MediaPreview extends JButtonIcon {

    private static final long serialVersionUID = 1L;
    private JPopupMenu popup;
    private Media media;

    /**
     * Create a new mediapreview for a given media
     *
     * @param m media to preview
     */
    public MediaPreview(Media m) {
        super(m.getTitle(), new ImageIcon(m.getImage().getScaledInstance(
                m.getImage().getHeight(null) > m.getImage().getWidth(null) ? Constants.PREVIEW_WIDTH * m.getImage().getWidth(null) / m.getImage().getHeight(null) : Constants.PREVIEW_WIDTH,
                m.getImage().getHeight(null) > m.getImage().getWidth(null) ? Constants.PREVIEW_WIDTH : Constants.PREVIEW_WIDTH * m.getImage().getHeight(null) / m.getImage().getWidth(null), Image.SCALE_FAST)));

        this.media = m;

        setSize(Constants.PREVIEW_WIDTH, Constants.PREVIEW_WIDTH);

        setOpaque(false);
        setContentAreaFilled(false);
        setBorderPainted(false);

        /*
           * Support for drag'n'drop
           */

        final Transferable transferable = new Transferable() {
            public boolean isDataFlavorSupported(DataFlavor flavor) {
                return flavor.isFlavorSerializedObjectType();
            }

            public DataFlavor[] getTransferDataFlavors() {
                try {
                    return new DataFlavor[]{new DataFlavor(
                            DataFlavor.javaJVMLocalObjectMimeType)};

                } catch (ClassNotFoundException e) {
                    e.printStackTrace();
                }

                return null;
            }

            public Object getTransferData(DataFlavor flavor) {
                return media;
            }
        };

        setTransferHandler(new TransferHandler() {
            private static final long serialVersionUID = 1L;

            protected Transferable createTransferable(JComponent c) {
                return transferable;
            }

            public int getSourceActions(JComponent c) {
                return COPY | MOVE;
            }

            protected void exportDone(JComponent source, Transferable data,
                                      int action) {
                super.exportDone(source, data, action);

                if (action == MOVE) {
                    Albums.getInstance().getSelected().removeMedia(media);
                }

                Albums.getInstance().refreshObservers(AlbumsMessage.SELECTION);
            }
        });

        addMouseMotionListener(new MouseMotionListener() {
            public void mouseMoved(MouseEvent e) {
            }

            public void mouseDragged(MouseEvent e) {
                getParent().setCursor(Cursor.getPredefinedCursor(Cursor.MOVE_CURSOR));
                if (Albums.getInstance().getMediaSearched() == null) {
                    if (Albums.getInstance().getSelected().getStatus() == ResourceStatus.ONLINE) {
                        JOptionPane
                                .showMessageDialog(
                                        null,
                                        "You have to synchronize the album to do drag'n'drop.",
                                        "Error", JOptionPane.ERROR_MESSAGE);
                    } else {
                        if (e.isControlDown()) {
                            getTransferHandler().exportAsDrag(
                                    MediaPreview.this, e, TransferHandler.COPY);
                        } else
                            getTransferHandler().exportAsDrag(
                                    MediaPreview.this, e, TransferHandler.MOVE);
                    }
                }
            }
        });
        // end drag'n drop code


        /*
           * Support for support/rename media contextual menu
           */
        popup = new JPopupMenu();

        JMenuItem mi = new JMenuItem("Delete");
        ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_DELETE));
        mi.setIcon(icon);
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                media.delete(true);
            }
        });
        popup.add(mi);

        mi = new JMenuItem("Rename");
        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_EDIT));
        mi.setIcon(icon);
        mi.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String name = JOptionPane.showInputDialog("Rename \""
                        + media.getTitle() + "\" :");
                if (name != null && !name.isEmpty()) {
                    media.setTitle(name);
                }
            }
        });
        popup.add(mi);

        addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                if (Albums.getInstance().getMediaSearched() == null) {
                    if (e.isPopupTrigger()) {
                        popup.show(MediaPreview.this, e.getX(), e.getY());
                    }
                }
            }

            // set the cursor to a hand
            @Override
            public void mouseEntered(MouseEvent e) {
                super.mouseEntered(e);
                getParent().setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
            }

            // set default cursor
            @Override
            public void mouseExited(MouseEvent e) {
                super.mouseExited(e);
                getParent().setCursor(Cursor.getDefaultCursor());

            }


        });
    }
}
