package fr.esial.seenshare.views;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Observable;
import java.util.Observer;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPopupMenu;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JToolBar;

import fr.esial.seenshare.controlers.CloseSoftware;
import fr.esial.seenshare.controlers.ConnectionListener;
import fr.esial.seenshare.controlers.ThemeListener;
import fr.esial.seenshare.models.Album;
import fr.esial.seenshare.models.Albums;
import fr.esial.seenshare.models.AlbumsMessage;
import fr.esial.seenshare.utils.Constants;
import fr.esial.seenshare.utils.JButtonIcon;

@SuppressWarnings("serial")
/**
 * View to display the buttons of the menu
 * This allows to add new album, folder, read about popup, ...
 */
public class MenusView extends JToolBar implements Observer {

    private Albums albums;

    private JButton btnConnect;
    private JButton btnAddFolder;
    private JButton btnAddAlbum;
    private JPopupMenu mTheme;
    private JButton btnTheme;
    private JButton btnHelp;
    private JButton btnQuit;
    private ImageIcon connectIcon;
    private ImageIcon disconnectIcon;

    /**
     * Create a new menus view
     *
     * @param parent window that contains the menus
     * @param a      list of all albums
     */
    public MenusView(final JFrame parent, Albums a) {
        super("Menu bar");

        albums = a;

        connectIcon = new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_CONNECT));
        disconnectIcon = new ImageIcon(getClass().getClassLoader().getResource(Constants.ICON_DISCONNECT));

        btnConnect = new JButtonIcon("Connect", connectIcon);
        btnConnect.setActionCommand("connect");
        btnConnect.addActionListener(new ConnectionListener());

        ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_ADD_FOLDER));
        btnAddFolder = new JButtonIcon("Add a folder", icon);
        btnAddFolder.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                JFileChooser chooser = new JFileChooser();
                chooser.setDialogTitle("Select a folder to scan");
                chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                int returnVal = chooser.showOpenDialog(null);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    File folder = chooser.getSelectedFile();
                    albums.addFolder(folder.getAbsolutePath());
                }
            }
        });

        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_ADD_ALBUM));
        btnAddAlbum = new JButtonIcon("New album", icon);
        btnAddAlbum.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                String name = JOptionPane.showInputDialog("Name of the album:");
                if (name != null && !name.isEmpty()) {
                    try {
                        albums.addAlbum(new Album(name));
                    } catch (URISyntaxException ex) {
                        ex.printStackTrace();
                    }
                }
            }
        });

        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_THEME));
        btnTheme = new JButtonIcon("Theme", icon);

        // Group the radio buttons.
        ButtonGroup themes = new ButtonGroup();

        mTheme = new JPopupMenu("Theme Switcher");

        String namesTheme[] = {"Acryl", "Aero", "Aluminium", "Bernstein",
                "Fast", "Graphite", "HiFi", "Luna", "McWin", "Mint", /* "Noire", */
                "Smart", "Quaqua"};
        final String classTheme[] = {
                "com.jtattoo.plaf.acryl.AcrylLookAndFeel",
                "com.jtattoo.plaf.aero.AeroLookAndFeel",
                "com.jtattoo.plaf.aluminium.AluminiumLookAndFeel",
                "com.jtattoo.plaf.bernstein.BernsteinLookAndFeel",
                "com.jtattoo.plaf.fast.FastLookAndFeel",
                "com.jtattoo.plaf.graphite.GraphiteLookAndFeel",
                "com.jtattoo.plaf.hifi.HiFiLookAndFeel",
                "com.jtattoo.plaf.luna.LunaLookAndFeel",
                "com.jtattoo.plaf.mcwin.McWinLookAndFeel",
                "com.jtattoo.plaf.mint.MintLookAndFeel",
                // "com.jtattoo.plaf.noire.NoireLookAndFeel",
                "com.jtattoo.plaf.smart.SmartLookAndFeel",
                "ch.randelshofer.quaqua.QuaquaLookAndFeel"};
        String selectedTheme = "Graphite";

        for (int i = 0; i < namesTheme.length; i++) {
            JRadioButtonMenuItem itTheme = new JRadioButtonMenuItem(
                    namesTheme[i]);
            if (namesTheme[i].equalsIgnoreCase(selectedTheme)) {
                itTheme.setSelected(true);
            }
            itTheme.addActionListener(new ThemeListener(parent, classTheme[i]));
            themes.add(itTheme);
            mTheme.add(itTheme);
        }

        btnTheme.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent ae) {
                mTheme.show(btnTheme, btnTheme.getWidth() / 2, btnTheme
                        .getHeight() / 2);
            }
        });

        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_HELP));
        btnHelp = new JButtonIcon("About", icon);
        btnHelp.addActionListener(new ActionListener() {

            @Override
            public void actionPerformed(ActionEvent e) {
                new DialogAbout();
            }
        });

        icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_QUIT));
        btnQuit = new JButtonIcon("Exit", icon);
        btnQuit.addActionListener(new CloseSoftware());

        add(btnConnect);
        add(btnAddFolder);
        add(btnAddAlbum);
        add(btnTheme);
        add(btnHelp);
        add(btnQuit);
    }

    @Override
    public void update(Observable o, Object arg) {
        if (o != null && o instanceof Albums) {
            if (arg != null && arg instanceof AlbumsMessage) {
                AlbumsMessage am = (AlbumsMessage) arg;
                if (am == AlbumsMessage.CONNECTION) {
                    btnConnect.setText("Disconnect");
                    btnConnect.setActionCommand("disconnect");
                    btnConnect.setIcon(disconnectIcon);
                } else if (am == AlbumsMessage.DISCONNECTION) {
                    btnConnect.setText("Connect");
                    btnConnect.setActionCommand("connect");
                    btnConnect.setIcon(connectIcon);
                }
            }
        }
    }
}
