package fr.esial.seenshare.views;

import com.google.gdata.data.photos.PhotoEntry;
import com.google.gdata.util.ServiceException;
import fr.esial.seenshare.controlers.CloseTabListener;
import fr.esial.seenshare.controlers.ImageEditButtonsActionListener;
import fr.esial.seenshare.controlers.PicturePanelMouseListener;
import fr.esial.seenshare.controlers.ZoomButtonsActionListener;
import fr.esial.seenshare.models.Media;
import fr.esial.seenshare.models.Photo;
import fr.esial.seenshare.utils.Constants;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.text.DecimalFormat;
import java.util.Observable;
import java.util.Observer;

/**
 * This view create a panel to display photos
 */
public class PhotoPanel extends MediaPanel implements Observer {

    public static final long serialVersionUID = 1L;

    protected PicturePanel picPanel;
    protected String picInfosText;
    protected Media media;

    /**
     * Create a new panel displaying a photo
     *
     * @param media photo to display
     * @param win   window where to draw
     */
    public PhotoPanel(Photo media, JFrame win) {
        super(media, win);

        this.media = media;
        media.addObserver(this);

        // Add Picture panel, main picture display
        picPanel = new PicturePanel(this, media);
        PicturePanelMouseListener mousePanelListener = new PicturePanelMouseListener();
        picPanel.addMouseListener(mousePanelListener);
        picPanel.addMouseMotionListener(mousePanelListener);
        picPanel.addMouseWheelListener(mousePanelListener);
        picPanel.addComponentListener(new PicPanelListener(picPanel)); //Listener defined under this class
        add(new JScrollPane(picPanel), BorderLayout.CENTER);

        // Setting up buttons top panel
        //Load ressources : Icons and Buttons listener
        ClassLoader cl = getClass().getClassLoader();

        ImageIcon iconClose = new ImageIcon(cl.getResource(Constants.ICON_CLOSE));
        ImageIcon iconZoomFit = new ImageIcon(cl.getResource(Constants.ICON_ZOOM_FIT));
        ImageIcon iconZoomFitWidth = new ImageIcon(cl.getResource(Constants.ICON_ZOOM_FIT_WIDTH));
        ImageIcon iconZoomOriginal = new ImageIcon(cl.getResource(Constants.ICON_ZOOM_ORIGINAL));
        ImageIcon iconZoomIn = new ImageIcon(cl.getResource(Constants.ICON_ZOOM_IN));
        ImageIcon iconZoomOut = new ImageIcon(cl.getResource(Constants.ICON_ZOOM_OUT));
        ImageIcon iconRotateLeft = new ImageIcon(cl.getResource(Constants.ICON_ROTATE_LEFT));
        ImageIcon iconRotateRight = new ImageIcon(cl.getResource(Constants.ICON_ROTATE_RIGHT));
        ImageIcon iconResize = new ImageIcon(cl.getResource(Constants.ICON_RESIZE));
        ImageIcon iconCrop = new ImageIcon(cl.getResource(Constants.ICON_CROP));
        ImageIcon iconPerson = new ImageIcon(cl.getResource(Constants.ICON_PERSON));
        ImageIcon iconSepia = new ImageIcon(cl.getResource(Constants.ICON_SEPIA));
        ImageIcon iconBW = new ImageIcon(cl.getResource(Constants.ICON_BW));
        ImageIcon iconBlur = new ImageIcon(cl.getResource(Constants.ICON_BLUR));

        JPanel buttonTopLeftPanel = new JPanel();
        JPanel buttonTopRightPanel = new JPanel();
        buttonTopRightPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));

        /** LEFT BUTTONS **/

        /**
         * Zoom buttons
         */
        ZoomButtonsActionListener zoomButtonsListener = new ZoomButtonsActionListener(picPanel);

        JButton zoomButtonFit = new JButton(iconZoomFit);
        zoomButtonFit.setToolTipText("Fit window");
        zoomButtonFit.setActionCommand("zoom_fit");
        zoomButtonFit.addActionListener(zoomButtonsListener);
        buttonTopLeftPanel.add(zoomButtonFit);

        JButton zoomButtonFitWidth = new JButton(iconZoomFitWidth);
        zoomButtonFitWidth.setToolTipText("Fit width");
        zoomButtonFitWidth.setActionCommand("zoom_fit_width");
        zoomButtonFitWidth.addActionListener(zoomButtonsListener);
        buttonTopLeftPanel.add(zoomButtonFitWidth);

        JButton zoomButtonOriginal = new JButton(iconZoomOriginal);
        zoomButtonOriginal.setToolTipText("Original size");
        zoomButtonOriginal.setActionCommand("zoom_original");
        zoomButtonOriginal.addActionListener(zoomButtonsListener);
        buttonTopLeftPanel.add(zoomButtonOriginal);

        JButton zoomButtonIn = new JButton(iconZoomIn);
        zoomButtonIn.setToolTipText("Zoom in");
        zoomButtonIn.setActionCommand("zoom_in");
        zoomButtonIn.addActionListener(zoomButtonsListener);
        buttonTopLeftPanel.add(zoomButtonIn);

        JButton zoomButtonOut = new JButton(iconZoomOut);
        zoomButtonOut.setToolTipText("Zoom out");
        zoomButtonOut.setActionCommand("zoom_out");
        zoomButtonOut.addActionListener(zoomButtonsListener);
        buttonTopLeftPanel.add(zoomButtonOut);

        JSeparator separator2 = new JSeparator(JSeparator.VERTICAL);
        separator2.setPreferredSize(new Dimension(20, 0));
        buttonTopLeftPanel.add(separator2);


        /**
         * Image edition buttons
         */
        ImageEditButtonsActionListener imageButtonListener = new ImageEditButtonsActionListener((Photo) media, mousePanelListener);

        JButton rotateButtonRight = new JButton(iconRotateRight);
        rotateButtonRight.setToolTipText("Rotate clockwise");
        rotateButtonRight.setActionCommand("rotate_right");
        rotateButtonRight.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(rotateButtonRight);

        JButton rotateButtonLeft = new JButton(iconRotateLeft);
        rotateButtonLeft.setToolTipText("Rotate counter-clockwise");
        rotateButtonLeft.setActionCommand("rotate_left");
        rotateButtonLeft.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(rotateButtonLeft);

        JSeparator separator3 = new JSeparator(JSeparator.VERTICAL);
        separator3.setPreferredSize(new Dimension(20, 0));
        buttonTopLeftPanel.add(separator3);

        JButton resizeButton = new JButton(iconResize);
        resizeButton.setToolTipText("Resize");
        resizeButton.setActionCommand("resize");
        resizeButton.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(resizeButton);

        JButton cropButton = new JButton(iconCrop);
        cropButton.setToolTipText("Crop");
        cropButton.setActionCommand("crop");
        cropButton.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(cropButton);

        JButton personTagButton = new JButton(iconPerson);
        personTagButton.setToolTipText("Tag a person");
        personTagButton.setActionCommand("person");
        personTagButton.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(personTagButton);

        JSeparator separator4 = new JSeparator(JSeparator.VERTICAL);
        separator4.setPreferredSize(new Dimension(20, 0));
        buttonTopLeftPanel.add(separator4);
        
        JButton sepiaButton = new JButton(iconSepia);
        sepiaButton.setToolTipText("Sepia");
        sepiaButton.setActionCommand("sepia");
        sepiaButton.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(sepiaButton);

        JButton bawButton = new JButton(iconBW);
        bawButton.setToolTipText("Black and White");
        bawButton.setActionCommand("bw");
        bawButton.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(bawButton);

        JButton blurButton = new JButton(iconBlur);
        blurButton.setToolTipText("Blur");
        blurButton.setActionCommand("blur");
        blurButton.addActionListener(imageButtonListener);
        buttonTopLeftPanel.add(blurButton);

        /** RIGHT BUTTONS **/

        /**
         * Button to close the current PhotoPanel tab
         */
        JButton buttonClose = new JButton(iconClose);
        buttonClose.addActionListener(new CloseTabListener(win, media));
        buttonTopRightPanel.add(buttonClose);


        buttonTopPanel.add(buttonTopLeftPanel, BorderLayout.WEST);
        buttonTopPanel.add(buttonTopRightPanel, BorderLayout.EAST);

        this.updateImageInfos();

        win.pack();

        setVisible(true);
        this.validate();
        picPanel.centerPic();
        picPanel.fitToScreen();
    }

    /**
     * Update the display of infos about the image (size)
     */
    public void updateImageInfos() {
        try {
            PhotoEntry pe = media.getEntry();
            
            Long size = pe.getSize();
            String photoSize;
            
            if (size != null) {
	            double pSize = (double)size;
	            DecimalFormat sizeFormat = new DecimalFormat("##0.0");
	            photoSize = pSize+"";
	            
	            //Format the media size in an human-readable format
	            if (pSize >= 1024) { //Kb
	            	pSize /= 1024;
	            	
	            	if (pSize >= 1024) { //Mb
	            		pSize /= 1024;
	            		photoSize = sizeFormat.format(pSize) +" Mb";
	            	}
	            	else
	            		photoSize = sizeFormat.format(pSize) +" Kb";
	            }
	            else
	        		photoSize = sizeFormat.format(pSize) +" b";
            }
            else
            	photoSize = "";
            
            picInfosText = media.getImage().getWidth(null) + "×"
                    + media.getImage().getHeight(null) + " px | "
                    + photoSize ;

        } catch (ServiceException e) {
            e.printStackTrace();
        }
    }

    /**
     * Update the display of zoom level
     */
    public void updateZoom() {
        bottomInfos.setText(picInfosText + " | " + (int) (picPanel.zoomLevel * 100) + "%");
    }


    @Override
    public void update(Observable o, Object arg) {
        picPanel.updateImage(media.getImage());
        this.updateImageInfos();
        this.updateZoom();
    }
}

/**
 * Controller that handles events on the PhotoPanel (resizing of the window),
 * triggering an update on the panel to fit the new panel size.
 */
class PicPanelListener implements ComponentListener {
    PicturePanel picPan;

    /**
     * Create a new listener
     *
     * @param p panel to listen
     */
    public PicPanelListener(PicturePanel p) {
        picPan = p;
    }

    public void componentShown(ComponentEvent arg0) {
    }

    public void componentMoved(ComponentEvent arg0) {
    }

    public void componentHidden(ComponentEvent arg0) {
    }

    public void componentResized(ComponentEvent arg0) {
        picPan.zoom(0);
    }
}
