package fr.esial.seenshare.views;

import java.awt.FlowLayout;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Observable;
import java.util.Observer;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.concurrent.Semaphore;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JFrame;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;

import fr.esial.seenshare.controlers.AddMediaListener;
import fr.esial.seenshare.models.*;
import fr.esial.seenshare.utils.Constants;

/**
 * View that display the preview of an album
 * It is the main display of the software : it displays a list of thumbnails of the medias
 */
public class PreviewView extends JPanel implements Observer {

    private static final long serialVersionUID = 1L;
    private JPopupMenu popup;
    private JFrame win;
    private final HashMap<Album, Semaphore[]> waiting = new HashMap<Album, Semaphore[]>();
    private final Executor executor;

    /**
     * Create a new view in the given window
     *
     * @param win window where to draw the view
     */
    public PreviewView(JFrame win) {
        super(new ModifiedFlowLayout(FlowLayout.LEFT));
        setBorder(BorderFactory.createTitledBorder("Content"));

        executor = Executors.newSingleThreadScheduledExecutor();

        this.win = win;
        popup = new JPopupMenu();

        JMenuItem mi = new JMenuItem("Add media");
        ImageIcon icon = new ImageIcon(getClass().getClassLoader().getResource(
                Constants.ICON_ADD));
        mi.setIcon(icon);
        mi.addActionListener(new AddMediaListener());
        popup.add(mi);

        addMouseListener(new MouseAdapter() {

            public void mousePressed(MouseEvent e) {
                if (e.isPopupTrigger()) {
                    popup.show(PreviewView.this, e.getX(), e.getY());
                }
            }
        });
    }

    /**
     * Update the preview view
     *
     * @param o   observable object
     * @param arg arg object
     */
    @Override
    public synchronized void update(Observable o, Object arg) {
        if (o != null && o instanceof Albums && arg != null && arg instanceof AlbumsMessage) {
            Albums albums = (Albums) o;
            AlbumsMessage am = (AlbumsMessage) arg;

            if (((am == AlbumsMessage.SELECTION || am == AlbumsMessage.SELECTION_AUTO) && albums.getSelected() != null) || am == AlbumsMessage.SEARCH) {
                if (am != AlbumsMessage.SEARCH) {
                    if (!waiting.containsKey(albums.getSelected()))
                        waiting.put(albums.getSelected(), new Semaphore[]{new Semaphore(2, true), new Semaphore(1, true)});

                    if (waiting.get(albums.getSelected())[0].tryAcquire())
                        executor.execute(new LoadMediaPreviewsThread(this, win, am, albums, waiting.get(albums.getSelected())));
                } else
                    new LoadMediaPreviewsThread(this, win, am, albums, null).start();
            }
        }
    }
}