package fr.esial.seenshare.views;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;

import fr.esial.seenshare.models.Photo;

/**
 * Display a popup to prompt the user for a resize value, given the current size of a photo
 */
public class ResizePopup extends JDialog {

    private static final long serialVersionUID = 9100096586515071327L;

    Photo photo;
    int pWidth;
    int pHeight;
    final JTextField newWidth;
    final JLabel newHeight;

    /**
     * Create a popup for the given photo
     *
     * @param m
     */
    public ResizePopup(Photo m) {
        photo = m;
        pWidth = photo.getImage().getWidth(null);
        pHeight = photo.getImage().getHeight(null);
        setModal(true);

        //
        // Message
        Container c = getContentPane();
        c.setLayout(new BorderLayout());

        c.add(new JLabel("Please enter the desired dimensions for the picture."),
                BorderLayout.NORTH);

        //
        // Form
        JPanel formRow = new JPanel();
        formRow.setLayout(new FlowLayout(FlowLayout.LEFT));

        newWidth = new JTextField(pWidth + "", 5);
        newHeight = new JLabel(pHeight + "");

        //newWidth.getDocument().addDocumentListener(new ResizeWidthRatioListener(this));

        formRow.add(new JLabel("New width: "));
        formRow.add(newWidth);
        formRow.add(new JLabel("pixels."));  /* × "));
			formRow.add(newHeight);
			formRow.add(new JLabel("px."));*/

        c.add(formRow, BorderLayout.CENTER);

        //
        // Buttons
        JPanel buttonsRow = new JPanel();
        buttonsRow.setLayout(new FlowLayout(FlowLayout.CENTER));

        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(ActionEvent e) {
                setVisible(false);
            }
        });
        buttonsRow.add(cancelButton);

        JButton okButton = new JButton("Resize");
        okButton.addActionListener(new ResizeButtonActionListener(photo, this));
        buttonsRow.add(okButton);

        c.add(buttonsRow, BorderLayout.SOUTH);


        pack();
        setLocationRelativeTo(getParent());

        setVisible(true);
    }
}


/**
 * Controller to handle clicks on the resize button
 */
class ResizeButtonActionListener implements ActionListener {
    Photo photo;
    ResizePopup parent;

    /**
     * Create a new controller
     *
     * @param photo  photo to resize
     * @param parent popup that launches event
     */
    public ResizeButtonActionListener(Photo photo, ResizePopup parent) {
        this.photo = photo;
        this.parent = parent;
    }

    public void actionPerformed(ActionEvent e) {
        photo.resize(Integer.parseInt(parent.newWidth.getText()));
        parent.setVisible(false);
    }
}

/**
 * Controller that listens the changes on the field of the popup, to update the value of the ratio
 */
class ResizeWidthRatioListener implements DocumentListener {

    ResizePopup parent;
    char source;

    public ResizeWidthRatioListener(ResizePopup parent) {
        this.parent = parent;
    }

    @Override
    public void changedUpdate(DocumentEvent e) {
        update();
    }

    @Override
    public void insertUpdate(DocumentEvent e) {
        update();
    }

    @Override
    public void removeUpdate(DocumentEvent e) {
        update();
    }

    public void update() {
        int nW = Integer.parseInt(parent.newWidth.getText());

        int newH = (parent.pHeight * nW) / parent.pWidth;
        parent.newWidth.setText(newH + "");
    }

}
