package fr.esial.seenshare.views;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.util.Observable;
import java.util.Observer;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.Timer;

import fr.esial.seenshare.models.Albums;
import fr.esial.seenshare.models.CurrentProgressStatus;

/**
 * View that displays a status bar w/ a progress bar and fields to launch searches
 */
public class StatusBar extends JPanel implements Observer {

    private static final long serialVersionUID = 1L;

    private JProgressBar progressBar;
    private CurrentProgressStatus status;

    /**
     * Create a status bar displaying a progress bar for the currentprogressstatus model
     *
     * @param s currentprogressstatus to display in a progressbar
     */
    public StatusBar(CurrentProgressStatus s) {
        this.status = s;

        this.setLayout(new BorderLayout());

        progressBar = new JProgressBar();
        progressBar.setPreferredSize(new Dimension(200, 10));
        progressBar.setMinimum(0);
        progressBar.setMaximum(99);

        this.add(progressBar, BorderLayout.WEST);

        JPanel search = new JPanel();

        String[] searchTypes = {"Title", "Description", "Tag", "Person"};

        JComboBox searchType = new JComboBox(searchTypes);
        JComboBox searchWhere = new SearchAlbumsView(Albums.getInstance());

        searchType.setToolTipText("By?");

        search.add(new JLabel("Search: "));
        search.add(searchType);
        search.add(searchWhere);
        search.add(new SearchView(Albums.getInstance(), searchType, searchWhere));

        this.add(search, BorderLayout.EAST);
    }

    /**
     * Clear the message of the progressbar
     */
    public void clearMessage() {
        progressBar.setString("");
        progressBar.setStringPainted(false);
    }

    /**
     * Update the display of the progressbar
     *
     * @param obs
     * @param ob
     */
    @Override
    public void update(Observable obs, Object ob) {
        if (status.isWorking()) {
            progressBar.setString(status.getMessage());
            progressBar.setStringPainted(true);
            progressBar.setValue((int) status.getProgress());
            progressBar.setIndeterminate(status.isIndeterminate());
        } else {
            int delay = 3000; //milliseconds
            ActionListener taskPerformer = new ActionListener() {
                public void actionPerformed(ActionEvent evt) {
                    clearMessage();
                }
            };
            new Timer(delay, taskPerformer).start();
        }
    }

}
