package fr.esial.seenshare.views;

//The contents of this file are subject to the Mozilla Public License Version 1.1
//(the "License"); you may not use this file except in compliance with the
//License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
//
//Software distributed under the License is distributed on an "AS IS" basis,
//WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//for the specific language governing rights and
//limitations under the License.
//
//The Original Code is "The Columba Project"
//
//The Initial Developers of the Original Code are Frederik Dietz and Timo Stich.
//Portions created by Frederik Dietz and Timo Stich are Copyright (C) 2003.
//
//All Rights Reserved.

import java.awt.*;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;

import javax.swing.*;

/**
 * View to display URL nicely
 */
public class URLLabel extends JLabel implements MouseListener {

    private static final long serialVersionUID = 1L;

    boolean entered = false;
    boolean mousehover;
    URL url;

    /**
     * Create a new URL
     *
     * @param url url to point to
     */
    public URLLabel(URL url) {
        this(url, url.toString());
    }

    /**
     * Create a new URL with a given name
     *
     * @param url url to point to
     * @param str name of the link
     */
    public URLLabel(URL url, String str) {
        super(str);

        setForeground(Color.blue);
        mousehover = true;

        this.url = url;

        addMouseListener(this);
    }

    /**
     * Get the URl
     *
     * @return url pointed by the view
     */
    public URL getUrl() {
        return url;
    }

    /**
     * Set a new url for the label
     *
     * @param url url to point to
     */
    public void setUrl(URL url) {
        this.url = url;
    }

    /**
     * Change the display of the label when mouse is over it
     *
     * @param e
     */
    public void mouseEntered(MouseEvent e) {
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
        entered = true;

        if (mousehover) {
            repaint();
        }
    }

    /**
     * Reset the display of the label when the mouse exits it
     *
     * @param e
     */
    public void mouseExited(MouseEvent e) {
        setCursor(Cursor.getDefaultCursor());
        entered = false;

        if (mousehover) {
            repaint();
        }
    }

    /**
     * Open the system web browser and go to the url when the users clicks on the label
     *
     * @param mouseEvent
     */
    @Override
    public void mouseClicked(MouseEvent mouseEvent) {
        URLLabel self = (URLLabel) mouseEvent.getSource();
        if (self.url == null)
            return;
        if (Desktop.isDesktopSupported()) {
            Desktop desktop = Desktop.getDesktop();
            if (desktop.isSupported(Desktop.Action.BROWSE))
                try {
                    desktop.browse(url.toURI());
                    return;
                } catch (Exception exp) {
                }
        }
        JOptionPane.showMessageDialog(this, "Cannot launch browser...\n Please, visit\n" + url.getRef(), "", JOptionPane.INFORMATION_MESSAGE);
        return;
    }

    public void mousePressed(MouseEvent e) {
    }

    public void mouseReleased(MouseEvent e) {
    }

    /**
     * Paint a nice label in blue, underlined when the users hover over it
     *
     * @param g
     */
    public void paint(Graphics g) {
        super.paint(g);

        if (entered || !mousehover) {
            Rectangle r = g.getClipBounds();

            g.drawLine(0, r.height - this.getFontMetrics(this.getFont()).getDescent(), this
                    .getFontMetrics(this.getFont()).stringWidth(this.getText()), r.height
                    - this.getFontMetrics(this.getFont()).getDescent());
        }
    }
}
